==> libtorrent-0.6.2.tar.gz: libtorrent-0.6.2/src/torrent/download.cc <==
// libTorrent - BitTorrent library
// Copyright (C) 2005, Jari Sundell
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Contact:  Jari Sundell <jaris@ifi.uio.no>
//
//           Skomakerveien 33
//           3185 Skoppum, NORWAY

#include "config.h"

#include "exceptions.h"
#include "download.h"
#include "peer_connection.h"

#include "tracker/tracker_control.h"
#include "data/hash_queue.h"
#include "data/hash_torrent.h"
#include "download/download_wrapper.h"

#include <algo/algo.h>
#include <sigc++/hide.h>

using namespace algo;

namespace torrent {

void
Download::open() {
  m_ptr->open();
}

void
Download::close() {
  if (m_ptr->get_main().is_active())
    stop();

  m_ptr->get_main().close();
}

void
Download::start() {
  m_ptr->get_main().start();
}

void
Download::stop() {
  m_ptr->stop();
}

void
Download::hash_check(bool resume) {
  if (resume)
    m_ptr->hash_load();

  m_ptr->get_hash_checker().start();
}

void
Download::hash_save() {
  m_ptr->hash_save();
}

bool
Download::is_open() {
  return m_ptr->get_main().is_open();
}

bool
Download::is_active() {
  return m_ptr->get_main().is_active();
}

bool
Download::is_tracker_busy() {
  return m_ptr->get_main().get_tracker().is_busy();
}

bool
Download::is_hash_checked() {
  return m_ptr->get_main().is_checked();
}

bool
Download::is_hash_checking() {
  return m_ptr->get_hash_checker().is_checking();
}

std::string
Download::get_name() {
  return m_ptr ? m_ptr->get_main().get_name() : "";
}

std::string
Download::get_hash() {
  return m_ptr ? m_ptr->get_main().get_hash() : "";
}

std::string
Download::get_id() {
  return m_ptr ? m_ptr->get_main().get_me().get_id() : "";
}

std::string
Download::get_root_dir() {
  return m_ptr->get_main().get_state().get_content().get_root_dir();
}

void
Download::set_root_dir(const std::string& dir) {
  if (is_open())
    throw client_error("Tried to call Download::set_root_dir(...) on an open download");

  m_ptr->get_main().get_state().get_content().set_root_dir(dir);
}

std::string
Download::get_ip() {
  return m_ptr->get_main().get_me().get_dns();
}

void
Download::set_ip(const std::string& ip) {
  m_ptr->get_main().get_me().set_dns(ip);
}

uint64_t
Download::get_bytes_up() {
  return m_ptr->get_main().get_net().get_rate_up().total();
}

uint64_t
Download::get_bytes_down() {
  return m_ptr->get_main().get_net().get_rate_down().total();
}

uint64_t
Download::get_bytes_done() {
  uint64_t a = 0;
 
  Delegator& d = m_ptr->get_main().get_net().get_delegator();

  std::for_each(d.get_chunks().begin(), d.get_chunks().end(),
		for_each_on(back_as_ref(),
			    if_on(call_member(&DelegatorPiece::is_finished),
				  
				  add_ref(a, call_member(call_member(&DelegatorPiece::get_piece),
							 &Piece::get_length)))));
  
  return a + m_ptr->get_main().get_state().get_content().get_bytes_completed();
}

uint64_t
Download::get_bytes_total() {
  return m_ptr->get_main().get_state().get_content().get_size();
}

uint32_t
Download::get_chunks_size() {
  return m_ptr->get_main().get_state().get_content().get_storage().get_chunk_size();
}

uint32_t
Download::get_chunks_done() {
  return m_ptr->get_main().get_state().get_content().get_chunks_completed();
}

uint32_t 
Download::get_chunks_total() {
  return m_ptr->get_main().get_state().get_chunk_total();
}

// Bytes per second.
uint32_t
Download::get_rate_up() {
  return m_ptr->get_main().get_net().get_rate_up().rate();
}

uint32_t
Download::get_rate_down() {
  return m_ptr->get_main().get_net().get_rate_down().rate();
}
  
const unsigned char*
Download::get_bitfield_data() {
  return (unsigned char*)m_ptr->get_main().get_state().get_content().get_bitfield().begin();
}

uint32_t
Download::get_bitfield_size() {
  return m_ptr->get_main().get_state().get_content().get_bitfield().size_bits();
}

uint32_t
Download::get_peers_min() {
  return m_ptr->get_main().get_state().get_settings().minPeers;
}

uint32_t
Download::get_peers_max() {
  return m_ptr->get_main().get_state().get_settings().maxPeers;
}

uint32_t
Download::get_peers_connected() {
  return m_ptr->get_main().get_net().get_connections().size();
}

uint32_t
Download::get_peers_not_connected() {
  return m_ptr->get_main().get_net().get_available_peers().size();
}

uint32_t
Download::get_uploads_max() {
  return m_ptr->get_main().get_state().get_settings().maxUploads;
}
  
uint64_t
Download::get_tracker_timeout() {
  return m_ptr->get_main().get_tracker().get_next_time().usec();
}

int16_t
Download::get_tracker_numwant() {
  return m_ptr->get_main().get_tracker().get_info().get_numwant();
}

void
Download::set_peers_min(uint32_t v) {
  if (v > 0 && v < 1000) {
    m_ptr->get_main().get_state().get_settings().minPeers = v;
    m_ptr->get_main().get_net().connect_peers();
  }
}

void
Download::set_peers_max(uint32_t v) {
  if (v > 0 && v < 1000) {
    m_ptr->get_main().get_state().get_settings().maxPeers = v;
    // TODO: Do disconnects here if nessesary
  }
}

void
Download::set_uploads_max(uint32_t v) {
  if (v > 0 && v < 1000) {
    m_ptr->get_main().get_state().get_settings().maxUploads = v;
    m_ptr->get_main().get_net().choke_balance();
  }
}

void
Download::set_tracker_timeout(uint64_t v, bool force) {
  m_ptr->get_main().get_tracker().set_next_time(v, force);
}

void
Download::set_tracker_numwant(int16_t n) {
  m_ptr->get_main().get_tracker().get_info().set_numwant(n);
}

Tracker
Download::get_tracker(uint32_t index) {
  if (index >= m_ptr->get_main().get_tracker().get_list().size())
    throw client_error("Client called Download::get_tracker(...) with out of range index");

  return m_ptr->get_main().get_tracker().get_list()[index];
}

uint32_t
Download::get_tracker_size() {
  return m_ptr->get_main().get_tracker().get_list().size();
}

Entry
Download::get_entry(uint32_t index) {
  if (index >= m_ptr->get_main().get_state().get_content().get_files().size())
    throw client_error("Client called Download::get_entry(...) with out of range index");

  return &m_ptr->get_main().get_state().get_content().get_files()[index];
}

uint32_t
Download::get_entry_size() {
  return m_ptr->get_main().get_state().get_content().get_files().size();
}

const Download::SeenVector&
Download::get_seen() {
  return m_ptr->get_main().get_state().get_bitfield_counter().field();
}

void
Download::update_priorities() {
  Priority& p = m_ptr->get_main().get_net().get_delegator().get_select().get_priority();
  Content& content = m_ptr->get_main().get_state().get_content();

  p.clear();

  uint64_t pos = 0;
  unsigned int cs = content.get_storage().get_chunk_size();

  for (Content::FileList::const_iterator i = content.get_files().begin(); i != content.get_files().end(); ++i) {
    unsigned int s = pos / cs;
    unsigned int e = i->get_size() ? (pos + i->get_size() + cs - 1) / cs : s;

    if (s != e)
      p.add((Priority::Type)i->get_priority(), s, e);

    pos += i->get_size();
  }

  std::for_each(m_ptr->get_main().get_net().get_connections().begin(), m_ptr->get_main().get_net().get_connections().end(),
		call_member(&PeerConnection::update_interested));
}

void
Download::peer_list(PList& pList) {
  std::for_each(m_ptr->get_main().get_net().get_connections().begin(),
		m_ptr->get_main().get_net().get_connections().end(),

		call_member(ref(pList),
			    &PList::push_back,
			    back_as_ptr()));
}

Peer
Download::peer_find(const std::string& id) {
  DownloadNet::ConnectionList::iterator itr =
    std::find_if(m_ptr->get_main().get_net().get_connections().begin(),
		 m_ptr->get_main().get_net().get_connections().end(),
		 
		 eq(ref(id),
		    call_member(call_member(&PeerConnectionBase::get_peer),
				&PeerInfo::get_id)));

  return itr != m_ptr->get_main().get_net().get_connections().end() ?
    *itr : NULL;
}

sigc::connection
Download::signal_download_done(Download::SlotVoid s) {
  return m_ptr->get_main().get_state().get_content().signal_download_done().connect(s);
}

sigc::connection
Download::signal_hash_done(Download::SlotVoid s) {
  return m_ptr->get_hash_checker().signal_torrent().connect(s);
}

sigc::connection
Download::signal_peer_connected(Download::SlotPeer s) {
  return m_ptr->get_main().get_net().signal_peer_connected().connect(s);
}

sigc::connection
Download::signal_peer_disconnected(Download::SlotPeer s) {
  return m_ptr->get_main().get_net().signal_peer_disconnected().connect(s);
}

sigc::connection
Download::signal_tracker_succeded(Download::SlotVoid s) {
  return m_ptr->get_main().get_tracker().signal_peers().connect(sigc::hide(s));
}

sigc::connection
Download::signal_tracker_failed(Download::SlotString s) {
  return m_ptr->get_main().get_tracker().signal_failed().connect(s);
}

sigc::connection
Download::signal_tracker_dump(Download::SlotIStream s) {
  return m_ptr->get_main().get_tracker().signal_dump().connect(s);
}

sigc::connection
Download::signal_chunk_passed(Download::SlotChunk s) {
  return m_ptr->get_main().get_state().signal_chunk_passed().connect(s);
}

sigc::connection
Download::signal_chunk_failed(Download::SlotChunk s) {
  return m_ptr->get_main().get_state().signal_chunk_failed().connect(s);
}

sigc::connection
Download::signal_network_log(SlotString s) {
  return m_ptr->get_main().get_net().signal_network_log().connect(s);
}

sigc::connection
Download::signal_storage_error(SlotString s) {
  return m_ptr->get_main().get_state().signal_storage_error().connect(s);
}

}

==> libtorrent-0.6.5.tar.gz: libtorrent-0.6.5/src/torrent/download.cc <==
// libTorrent - BitTorrent library
// Copyright (C) 2005, Jari Sundell
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Contact:  Jari Sundell <jaris@ifi.uio.no>
//
//           Skomakerveien 33
//           3185 Skoppum, NORWAY

#include "config.h"

#include "exceptions.h"
#include "download.h"
#include "peer_connection.h"

#include "tracker/tracker_control.h"
#include "data/hash_queue.h"
#include "data/hash_torrent.h"
#include "download/download_wrapper.h"

#include <rak/functional.h>
#include <sigc++/hide.h>

namespace torrent {

void
Download::open() {
  m_ptr->open();
}

void
Download::close() {
  if (m_ptr->get_main().is_active())
    stop();

  m_ptr->get_main().close();
}

void
Download::start() {
  m_ptr->get_main().start();
}

void
Download::stop() {
  m_ptr->stop();
}

void
Download::hash_check(bool resume) {
  if (resume)
    m_ptr->hash_load();

  m_ptr->get_hash_checker().start();
}

void
Download::hash_save() {
  m_ptr->hash_save();
}

bool
Download::is_open() {
  return m_ptr->get_main().is_open();
}

bool
Download::is_active() {
  return m_ptr->get_main().is_active();
}

bool
Download::is_tracker_busy() {
  return m_ptr->get_main().get_tracker().is_busy();
}

bool
Download::is_hash_checked() {
  return m_ptr->get_main().is_checked();
}

bool
Download::is_hash_checking() {
  return m_ptr->get_hash_checker().is_checking();
}

std::string
Download::get_name() {
  return m_ptr ? m_ptr->get_main().get_name() : "";
}

std::string
Download::get_hash() {
  return m_ptr ? m_ptr->get_main().get_hash() : "";
}

std::string
Download::get_id() {
  return m_ptr ? m_ptr->get_main().get_me().get_id() : "";
}

std::string
Download::get_root_dir() {
  return m_ptr->get_main().get_state().get_content().get_root_dir();
}

void
Download::set_root_dir(const std::string& dir) {
  if (is_open())
    throw client_error("Tried to call Download::set_root_dir(...) on an open download");

  m_ptr->get_main().get_state().get_content().set_root_dir(dir);
}

uint64_t
Download::get_bytes_up() {
  return m_ptr->get_main().get_net().get_write_rate().total();
}

uint64_t
Download::get_bytes_down() {
  return m_ptr->get_main().get_net().get_read_rate().total();
}

uint64_t
Download::get_bytes_done() {
  uint64_t a = 0;
 
  Delegator& d = m_ptr->get_main().get_net().get_delegator();

  for (Delegator::Chunks::iterator itr1 = d.get_chunks().begin(), last1 = d.get_chunks().end(); itr1 != last1; ++itr1)
    for (DelegatorChunk::iterator itr2 = (*itr1)->begin(), last2 = (*itr1)->end(); itr2 != last2; ++itr2)
      if (itr2->is_finished())
	a += itr2->get_piece().get_length();
  
  return a + m_ptr->get_main().get_state().get_content().get_bytes_completed();
}

uint64_t
Download::get_bytes_total() {
  return m_ptr->get_main().get_state().get_content().get_size();
}

uint32_t
Download::get_chunks_size() {
  return m_ptr->get_main().get_state().get_content().get_storage().get_chunk_size();
}

uint32_t
Download::get_chunks_done() {
  return m_ptr->get_main().get_state().get_content().get_chunks_completed();
}

uint32_t 
Download::get_chunks_total() {
  return m_ptr->get_main().get_state().get_chunk_total();
}

// Bytes per second.
uint32_t
Download::get_rate_up() {
  return m_ptr->get_main().get_net().get_write_rate().rate();
}

uint32_t
Download::get_rate_down() {
  return m_ptr->get_main().get_net().get_read_rate().rate();
}
  
const unsigned char*
Download::get_bitfield_data() {
  return (unsigned char*)m_ptr->get_main().get_state().get_content().get_bitfield().begin();
}

uint32_t
Download::get_bitfield_size() {
  return m_ptr->get_main().get_state().get_content().get_bitfield().size_bits();
}

uint32_t
Download::get_peers_min() {
  return m_ptr->get_main().get_state().get_settings().minPeers;
}

uint32_t
Download::get_peers_max() {
  return m_ptr->get_main().get_state().get_settings().maxPeers;
}

uint32_t
Download::get_peers_connected() {
  return m_ptr->get_main().get_net().get_connections().size();
}

uint32_t
Download::get_peers_not_connected() {
  return m_ptr->get_main().get_net().get_available_peers().size();
}

uint32_t
Download::get_uploads_max() {
  return m_ptr->get_main().get_net().get_choke_manager().get_max_unchoked();
}
  
uint64_t
Download::get_tracker_timeout() {
  return m_ptr->get_main().get_tracker().get_next_time().usec();
}

int16_t
Download::get_tracker_numwant() {
  return m_ptr->get_main().get_tracker().get_info().get_numwant();
}

void
Download::set_peers_min(uint32_t v) {
  if (v >= 0 && v < (1 << 16)) {
    m_ptr->get_main().get_state().get_settings().minPeers = v;
    m_ptr->get_main().get_net().connect_peers();
  }
}

void
Download::set_peers_max(uint32_t v) {
  if (v >= 0 && v < (1 << 16)) {
    m_ptr->get_main().get_state().get_settings().maxPeers = v;
    // TODO: Do disconnects here if nessesary
  }
}

void
Download::set_uploads_max(uint32_t v) {
  if (v >= 0 && v < (1 << 16)) {
    m_ptr->get_main().get_net().get_choke_manager().set_max_unchoked(v);
    m_ptr->get_main().get_net().choke_balance();
  }
}

void
Download::set_tracker_timeout(uint64_t v, bool force) {
  m_ptr->get_main().get_tracker().set_next_time(v, force);
}

void
Download::set_tracker_numwant(int16_t n) {
  m_ptr->get_main().get_tracker().get_info().set_numwant(n);
}

Tracker
Download::get_tracker(uint32_t index) {
  if (index >= m_ptr->get_main().get_tracker().get_list().size())
    throw client_error("Client called Download::get_tracker(...) with out of range index");

  return m_ptr->get_main().get_tracker().get_list()[index];
}

uint32_t
Download::get_tracker_size() {
  return m_ptr->get_main().get_tracker().get_list().size();
}

Entry
Download::get_entry(uint32_t index) {
  if (index >= m_ptr->get_main().get_state().get_content().get_files().size())
    throw client_error("Client called Download::get_entry(...) with out of range index");

  return &m_ptr->get_main().get_state().get_content().get_files()[index];
}

uint32_t
Download::get_entry_size() {
  return m_ptr->get_main().get_state().get_content().get_files().size();
}

const Download::SeenVector&
Download::get_seen() {
  return m_ptr->get_main().get_state().get_bitfield_counter().field();
}

void
Download::update_priorities() {
  Priority& p = m_ptr->get_main().get_net().get_delegator().get_select().get_priority();
  Content& content = m_ptr->get_main().get_state().get_content();

  p.clear();

  uint64_t pos = 0;
  unsigned int cs = content.get_storage().get_chunk_size();

  for (Content::FileList::const_iterator i = content.get_files().begin(); i != content.get_files().end(); ++i) {
    unsigned int s = pos / cs;
    unsigned int e = i->get_size() ? (pos + i->get_size() + cs - 1) / cs : s;

    if (s != e)
      p.add((Priority::Type)i->get_priority(), s, e);

    pos += i->get_size();
  }

  std::for_each(m_ptr->get_main().get_net().get_connections().begin(),
		m_ptr->get_main().get_net().get_connections().end(),
		std::mem_fun(&PeerConnection::update_interested));
}

void
Download::peer_list(PList& pList) {
  std::for_each(m_ptr->get_main().get_net().get_connections().begin(),
		m_ptr->get_main().get_net().get_connections().end(),
		rak::bind1st(std::mem_fun<void,PList>(&PList::push_back), &pList));
}

Peer
Download::peer_find(const std::string& id) {
  DownloadNet::ConnectionList::iterator itr =
    std::find_if(m_ptr->get_main().get_net().get_connections().begin(),
		 m_ptr->get_main().get_net().get_connections().end(),
		 
		 rak::equal(id, rak::on(std::mem_fun(&PeerConnectionBase::get_peer),
					std::mem_fun_ref(&PeerInfo::get_id))));

  return itr != m_ptr->get_main().get_net().get_connections().end() ?
    *itr : NULL;
}

sigc::connection
Download::signal_download_done(Download::SlotVoid s) {
  return m_ptr->get_main().get_state().get_content().signal_download_done().connect(s);
}

sigc::connection
Download::signal_hash_done(Download::SlotVoid s) {
  return m_ptr->get_hash_checker().signal_torrent().connect(s);
}

sigc::connection
Download::signal_peer_connected(Download::SlotPeer s) {
  return m_ptr->get_main().get_net().signal_peer_connected().connect(s);
}

sigc::connection
Download::signal_peer_disconnected(Download::SlotPeer s) {
  return m_ptr->get_main().get_net().signal_peer_disconnected().connect(s);
}

sigc::connection
Download::signal_tracker_succeded(Download::SlotVoid s) {
  return m_ptr->get_main().get_tracker().signal_peers().connect(sigc::hide(s));
}

sigc::connection
Download::signal_tracker_failed(Download::SlotString s) {
  return m_ptr->get_main().get_tracker().signal_failed().connect(s);
}

sigc::connection
Download::signal_tracker_dump(Download::SlotIStream s) {
  return m_ptr->get_main().get_tracker().signal_dump().connect(s);
}

sigc::connection
Download::signal_chunk_passed(Download::SlotChunk s) {
  return m_ptr->get_main().get_state().signal_chunk_passed().connect(s);
}

sigc::connection
Download::signal_chunk_failed(Download::SlotChunk s) {
  return m_ptr->get_main().get_state().signal_chunk_failed().connect(s);
}

sigc::connection
Download::signal_network_log(SlotString s) {
  return m_ptr->get_main().get_net().signal_network_log().connect(s);
}

sigc::connection
Download::signal_storage_error(SlotString s) {
  return m_ptr->get_main().get_state().signal_storage_error().connect(s);
}

}
